package com.crm.manager.service.risk;

import com.crm.rely.backend.core.dto.base.BaseResultDto;
import com.crm.rely.backend.core.dto.base.ResultWithPagerDto;
import com.crm.rely.backend.core.dto.risk.riskimport.trade.RiskImportTradeDto;
import com.crm.rely.backend.core.dto.risk.riskimport.trade.record.RiskImportTradeRecordDto;
import com.crm.rely.backend.core.entity.base.SingleLongEntity;
import com.crm.rely.backend.core.entity.risk.riskimport.trade.RiskImportTradeAddEntity;
import com.crm.rely.backend.core.entity.risk.riskimport.trade.RiskImportTradeDeleteEntity;
import com.crm.rely.backend.core.entity.risk.riskimport.trade.RiskImportTradeSearchPageEntity;
import com.crm.rely.backend.core.entity.risk.riskimport.trade.RiskImportTradeUpdateEntity;
import com.crm.rely.backend.core.entity.risk.riskimport.trade.record.*;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;

@FeignClient(name = "RISKBBI-SERVICE", path = "/risk/import/trade/manager")
public interface RiskImportTradeFeignService {

    @PostMapping("/add")
    BaseResultDto add(RiskImportTradeAddEntity entity) throws Exception;

    @PostMapping("/update")
    BaseResultDto update(RiskImportTradeUpdateEntity entity) throws Exception;

    @PostMapping("/delete")
    BaseResultDto delete(RiskImportTradeDeleteEntity entity) throws Exception;

    @PostMapping("/search/list")
    ResultWithPagerDto<RiskImportTradeDto> searchPage(RiskImportTradeSearchPageEntity entity) throws Exception;

    @PostMapping("/search/single")
    BaseResultDto<RiskImportTradeDto> searchSingle(SingleLongEntity entity) throws Exception;

    @PostMapping("/add/record/batch")
    BaseResultDto addRecordBatch(RiskImportTradeRecordAddBatchEntity entity) throws Exception;

    @PostMapping("/add/record/single")
    BaseResultDto addRecordSingle(RiskImportTradeRecordAddSingleEntity entity) throws Exception;

    @PostMapping("/update/record")
    BaseResultDto updateRecord(RiskImportTradeRecordUpdateEntity entity) throws Exception;

    @PostMapping("/delete/record")
    BaseResultDto deleteRecord(RiskImportTradeRecordDeleteEntity entity) throws Exception;

    @PostMapping("/search/list/record")
    ResultWithPagerDto<RiskImportTradeRecordDto> searchRecordPage(RiskImportTradeRecordSearchPageEntity entity) throws Exception;

    @PostMapping("/search/single/record")
    BaseResultDto<RiskImportTradeRecordDto> searchRecordSingle(SingleLongEntity entity) throws Exception;

}
